@@ -3,13 +3,7 @@ Decompressor. Algorithm Ported from OPSD code (Decomp.asm) for Efi and Tiano
 compress algorithm.
 
 Copyright (c) 2004 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 --*/
 
@@ -254,10 +248,11 @@ Returns:
 
     if (Len <= TableBits) {
 
+      if (Start[Len] >= NextCode || NextCode > MaxTableLength){
+        return (UINT16) BAD_TABLE;
+      }
+
       for (Index = Start[Len]; Index < NextCode; Index++) {
-        if (Index >= MaxTableLength) {
-          return (UINT16) BAD_TABLE;
-        }
         Table[Index] = Char;
       }
 
@@ -747,7 +742,7 @@ Arguments:
 
 Returns:
 
-  EFI_SUCCESS           - Decompression is successfull
+  EFI_SUCCESS           - Decompression is successful
   EFI_INVALID_PARAMETER - The source data is corrupted
 
 --*/
@@ -902,7 +897,7 @@ Arguments:
 
 Returns:
 
-  EFI_SUCCESS           - Decompression is successfull
+  EFI_SUCCESS           - Decompression is successful
   EFI_INVALID_PARAMETER - The source data is corrupted
 
 --*/
@@ -937,7 +932,7 @@ Arguments:
 
 Returns:
 
-  EFI_SUCCESS           - Decompression is successfull
+  EFI_SUCCESS           - Decompression is successful
   EFI_INVALID_PARAMETER - The source data is corrupted
 
 --*/
