@@ -2,13 +2,7 @@
   Definition for Device Path Tool.
 
 Copyright (c) 2017 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -178,7 +172,7 @@ int main(int argc, CHAR8 *argv[])
   }
   Str16 = (CHAR16 *)malloc(1024);
   if (Str16 == NULL) {
-    fprintf(stderr, "Resource, memory cannot be allcoated");
+    fprintf(stderr, "Resource, memory cannot be allocated");
     return STATUS_ERROR;
   }
   Ascii2UnicodeString(Str, Str16);
