@@ -1,24 +1,19 @@
 ## @file
 # This file is used to define common static strings and global data used by UPT
 #
-# Copyright (c) 2011, Intel Corporation. All rights reserved.<BR>
+# Copyright (c) 2011 - 2018, Intel Corporation. All rights reserved.<BR>
 #
-# This program and the accompanying materials are licensed and made available 
-# under the terms and conditions of the BSD License which accompanies this 
-# distribution. The full text of the license may be found at 
-# http://opensource.org/licenses/bsd-license.php
-#
-# THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-# WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+# SPDX-License-Identifier: BSD-2-Clause-Patent
 
 '''
 GlobalData
 '''
 
 #
-# The workspace directory 
+# The workspace directory
 #
 gWORKSPACE = '.'
+gPACKAGE_PATH = None
 
 #
 # INF module directory
@@ -42,7 +37,7 @@ gINVALID_MODULE_FILE = gUPT_DIR + r"Invalid_Modules.log"
 gCONTENT_FILE = "dist.content"
 
 #
-# File name for XML file in the distibution
+# File name for XML file in the distribution
 #
 gDESC_FILE = 'dist.pkg'
 
@@ -86,14 +81,30 @@ gWARNING_AS_ERROR = False
 #
 # Used to specify the temp directory to hold the unpacked distribution files
 #
-gUNPACK_DIR = None
+gUNPACK_DIR = []
 
 #
 # Flag used to mark whether the INF file is Binary INF or not.
 #
 gIS_BINARY_INF = False
+
+#
+# Used by FileHook module.
+#
+gRECOVERMGR = None
+
+#
+# Used by PCD parser
+#
+gPackageDict = {}
+
 #
 # Used by Library instance parser
 # {FilePath: FileObj}
 #
-gLIBINSTANCEDICT = {}
\ No newline at end of file
+gLIBINSTANCEDICT = {}
+
+#
+# Store the list of DIST
+#
+gTO_BE_INSTALLED_DIST_LIST = []
