@@ -3,13 +3,7 @@
   The EHCI register operation routines.
 
 Copyright (c) 2007 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -519,7 +513,7 @@ UhciCheckTdStatus (
       // terminate the transfer
       //
       if (!IsLow && (TdHw->ShortPacket == 1) && (Len < Td->DataLen)) {
-        DEBUG ((EFI_D_VERBOSE, "UhciCheckTdStatus: short packet read occured\n"));
+        DEBUG ((DEBUG_VERBOSE, "UhciCheckTdStatus: short packet read occurred\n"));
 
         Finished = TRUE;
         goto ON_EXIT;
@@ -596,7 +590,7 @@ UhciExecuteTransfer (
     Finished = UhciCheckTdStatus (Uhc, Td, IsLow, QhResult);
 
     //
-    // Transfer is OK or some error occured (TD inactive)
+    // Transfer is OK or some error occurred (TD inactive)
     //
     if (Finished) {
       break;
