@@ -2,13 +2,7 @@
   Header file for USB Keyboard Driver's Data Structures.
 
 Copyright (c) 2004 - 2017, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 #ifndef _EFI_USB_KB_H_
@@ -383,7 +377,7 @@ UsbKeyboardComponentNameGetControllerName (
 // Functions of Simple Text Input Protocol
 //
 /**
-  Reset the input device and optionaly run diagnostics
+  Reset the input device and optionally run diagnostics
 
   There are 2 types of reset for USB keyboard.
   For non-exhaustive reset, only keyboard buffer is cleared.
@@ -412,8 +406,8 @@ USBKeyboardReset (
                                information for the key that was pressed.
 
   @retval EFI_SUCCESS          The keystroke information was returned.
-  @retval EFI_NOT_READY        There was no keystroke data availiable.
-  @retval EFI_DEVICE_ERROR     The keydtroke information was not returned due to
+  @retval EFI_NOT_READY        There was no keystroke data available.
+  @retval EFI_DEVICE_ERROR     The keystroke information was not returned due to
                                hardware errors.
 
 **/
@@ -513,7 +507,7 @@ USBKeyboardSetState (
   @param  NotifyHandle                Points to the unique handle assigned to the registered notification.
 
   @retval EFI_SUCCESS                 The notification function was registered successfully.
-  @retval EFI_OUT_OF_RESOURCES        Unable to allocate resources for necesssary data structures.
+  @retval EFI_OUT_OF_RESOURCES        Unable to allocate resources for necessary data structures.
   @retval EFI_INVALID_PARAMETER       KeyData or NotifyHandle or KeyNotificationFunction is NULL.
 
 **/
@@ -580,7 +574,7 @@ KbdFreeNotifyList (
   @param  InputData         A pointer to keystroke data for the key that was pressed.
 
   @retval TRUE              Key pressed matches a registered key.
-  @retval FLASE             Key pressed does not matche a registered key.
+  @retval FALSE             Key pressed does not match a registered key.
 
 **/
 BOOLEAN
