@@ -1,18 +1,13 @@
 /** @file
   The header file for Boot Script Executer module.
-  
-  This driver is dispatched by Dxe core and the driver will reload itself to ACPI NVS memory 
-  in the entry point. The functionality is to interpret and restore the S3 boot script 
-  
-Copyright (c) 2006 - 2012, Intel Corporation. All rights reserved.<BR>
 
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
+  This driver is dispatched by Dxe core and the driver will reload itself to ACPI reserved memory
+  in the entry point. The functionality is to interpret and restore the S3 boot script
 
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
+Copyright (c) 2017, AMD Incorporated. All rights reserved.<BR>
+
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 #ifndef _BOOT_SCRIPT_EXECUTOR_H_
@@ -36,16 +31,23 @@ WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
 #include <Library/DebugAgentLib.h>
 #include <Library/LockBoxLib.h>
 #include <Library/CpuExceptionHandlerLib.h>
+#include <Library/DevicePathLib.h>
+#include <Library/DxeServicesTableLib.h>
 
 #include <Guid/AcpiS3Context.h>
 #include <Guid/BootScriptExecutorVariable.h>
-#include <Guid/EventGroup.h>
+#include <Guid/MemoryProfile.h>
+
+#include <Protocol/DxeSmmReadyToLock.h>
 #include <IndustryStandard/Acpi.h>
+
+#define PAGING_1G_ADDRESS_MASK_64  0x000FFFFFC0000000ull
+
 /**
   a ASM function to transfer control to OS.
-  
+
   @param  S3WakingVector  The S3 waking up vector saved in ACPI Facs table
-  @param  AcpiLowMemoryBase a buffer under 1M which could be used during the transfer             
+  @param  AcpiLowMemoryBase a buffer under 1M which could be used during the transfer
 **/
 VOID
 AsmTransferControl (
@@ -54,9 +56,9 @@ AsmTransferControl (
   );
 /**
   a 32bit ASM function to transfer control to OS.
-  
+
   @param  S3WakingVector  The S3 waking up vector saved in ACPI Facs table
-  @param  AcpiLowMemoryBase a buffer under 1M which could be used during the transfer             
+  @param  AcpiLowMemoryBase a buffer under 1M which could be used during the transfer
 **/
 VOID
 AsmTransferControl32 (
@@ -71,17 +73,19 @@ AsmTransferControl16 (
   VOID
   );
 /**
-  Set a IDT entry for interrupt vector 3 for debug purpose.  
-  
-  @param  AcpiS3Context  a pointer to a structure of ACPI_S3_CONTEXT  
-              
+  Set a IDT entry for interrupt vector 3 for debug purpose.
+
+  @param  AcpiS3Context  a pointer to a structure of ACPI_S3_CONTEXT
+
 **/
 VOID
-SetIdtEntry ( 
+SetIdtEntry (
   IN ACPI_S3_CONTEXT     *AcpiS3Context
   );
 
 extern UINT32 AsmFixAddress16;
 extern UINT32 AsmJmpAddr32;
+extern BOOLEAN mPage1GSupport;
+extern UINT64 mAddressEncMask;
 
 #endif //_BOOT_SCRIPT_EXECUTOR_H_
