@@ -3,13 +3,7 @@
 
   Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
   Portions copyright (c) 2008 - 2009, Apple Inc. All rights reserved.<BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php.
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -349,65 +343,6 @@ UnicodeSPrintAsciiFormat (
   return NumberOfPrinted;
 }
 
-#ifndef DISABLE_NEW_DEPRECATED_INTERFACES
-
-/**
-  [ATTENTION] This function is deprecated for security reason.
-
-  Converts a decimal value to a Null-terminated Unicode string.
-
-  Converts the decimal number specified by Value to a Null-terminated Unicode
-  string specified by Buffer containing at most Width characters. No padding of spaces
-  is ever performed. If Width is 0 then a width of MAXIMUM_VALUE_CHARACTERS is assumed.
-  The number of Unicode characters in Buffer is returned not including the Null-terminator.
-  If the conversion contains more than Width characters, then only the first
-  Width characters are returned, and the total number of characters
-  required to perform the conversion is returned.
-  Additional conversion parameters are specified in Flags.
-
-  The Flags bit LEFT_JUSTIFY is always ignored.
-  All conversions are left justified in Buffer.
-  If Width is 0, PREFIX_ZERO is ignored in Flags.
-  If COMMA_TYPE is set in Flags, then PREFIX_ZERO is ignored in Flags, and commas
-  are inserted every 3rd digit starting from the right.
-  If RADIX_HEX is set in Flags, then the output buffer will be
-  formatted in hexadecimal format.
-  If Value is < 0 and RADIX_HEX is not set in Flags, then the fist character in Buffer is a '-'.
-  If PREFIX_ZERO is set in Flags and PREFIX_ZERO is not being ignored,
-  then Buffer is padded with '0' characters so the combination of the optional '-'
-  sign character, '0' characters, digit characters for Value, and the Null-terminator
-  add up to Width characters.
-  If both COMMA_TYPE and RADIX_HEX are set in Flags, then ASSERT().
-  If Buffer is NULL, then ASSERT().
-  If Buffer is not aligned on a 16-bit boundary, then ASSERT().
-  If unsupported bits are set in Flags, then ASSERT().
-  If both COMMA_TYPE and RADIX_HEX are set in Flags, then ASSERT().
-  If Width >= MAXIMUM_VALUE_CHARACTERS, then ASSERT()
-
-  @param  Buffer  The pointer to the output buffer for the produced Null-terminated
-                  Unicode string.
-  @param  Flags   The bitmask of flags that specify left justification, zero pad, and commas.
-  @param  Value   The 64-bit signed value to convert to a string.
-  @param  Width   The maximum number of Unicode characters to place in Buffer, not including
-                  the Null-terminator.
-
-  @return The number of Unicode characters in Buffer not including the Null-terminator.
-
-**/
-UINTN
-EFIAPI
-UnicodeValueToString (
-  IN OUT CHAR16  *Buffer,
-  IN UINTN       Flags,
-  IN INT64       Value,
-  IN UINTN       Width
-  )
-{
-  ASSERT_UNICODE_BUFFER(Buffer);
-  return BasePrintLibConvertValueToString ((CHAR8 *)Buffer, Flags, Value, Width, 2);
-}
-
-#endif
 
 /**
   Converts a decimal value to a Null-terminated Unicode string.
@@ -786,65 +721,6 @@ AsciiSPrintUnicodeFormat (
   return NumberOfPrinted;
 }
 
-
-#ifndef DISABLE_NEW_DEPRECATED_INTERFACES
-
-/**
-  [ATTENTION] This function is deprecated for security reason.
-
-  Converts a decimal value to a Null-terminated ASCII string.
-
-  Converts the decimal number specified by Value to a Null-terminated ASCII string
-  specified by Buffer containing at most Width characters. No padding of spaces
-  is ever performed.
-  If Width is 0 then a width of  MAXIMUM_VALUE_CHARACTERS is assumed.
-  The number of ASCII characters in Buffer is returned not including the Null-terminator.
-  If the conversion contains more than Width characters, then only the first Width
-  characters are returned, and the total number of characters required to perform
-  the conversion is returned.
-  Additional conversion parameters are specified in Flags.
-  The Flags bit LEFT_JUSTIFY is always ignored.
-  All conversions are left justified in Buffer.
-  If Width is 0, PREFIX_ZERO is ignored in Flags.
-  If COMMA_TYPE is set in Flags, then PREFIX_ZERO is ignored in Flags, and commas
-  are inserted every 3rd digit starting from the right.
-  If RADIX_HEX is set in Flags, then the output buffer will be
-  formatted in hexadecimal format.
-  If Value is < 0 and RADIX_HEX is not set in Flags, then the fist character in Buffer is a '-'.
-  If PREFIX_ZERO is set in Flags and PREFIX_ZERO is not being ignored,
-  then Buffer is padded with '0' characters so the combination of the optional '-'
-  sign character, '0' characters, digit characters for Value, and the Null-terminator
-  add up to Width characters.
-
-  If Buffer is NULL, then ASSERT().
-  If unsupported bits are set in Flags, then ASSERT().
-  If both COMMA_TYPE and RADIX_HEX are set in Flags, then ASSERT().
-  If Width >= MAXIMUM_VALUE_CHARACTERS, then ASSERT()
-
-  @param  Buffer  The pointer to the output buffer for the produced Null-terminated
-                  ASCII string.
-  @param  Flags   The bitmask of flags that specify left justification, zero pad, and commas.
-  @param  Value   The 64-bit signed value to convert to a string.
-  @param  Width   The maximum number of ASCII characters to place in Buffer, not including
-                  the Null-terminator.
-
-  @return The number of ASCII characters in Buffer not including the Null-terminator.
-
-**/
-UINTN
-EFIAPI
-AsciiValueToString (
-  OUT CHAR8      *Buffer,
-  IN  UINTN      Flags,
-  IN  INT64      Value,
-  IN  UINTN      Width
-  )
-{
-  return BasePrintLibConvertValueToString (Buffer, Flags, Value, Width, 1);
-}
-
-#endif
-
 /**
   Converts a decimal value to a Null-terminated Ascii string.
 
