@@ -2,13 +2,7 @@
   Helper functions for configuring or getting the parameters relating to HTTP Boot.
 
 Copyright (c) 2016 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -558,7 +552,7 @@ HttpBootConfigFormInit (
 
   CallbackInfo = &Private->CallbackInfo;
 
-  if (CallbackInfo->Initilized) {
+  if (CallbackInfo->Initialized) {
     return EFI_SUCCESS;
   }
 
@@ -638,7 +632,7 @@ HttpBootConfigFormInit (
     FreePool (MacString);
     FreePool (OldMenuString);
 
-    CallbackInfo->Initilized = TRUE;
+    CallbackInfo->Initialized = TRUE;
     return EFI_SUCCESS;
   }
 
