@@ -2,22 +2,62 @@
   Member functions of EFI_SHELL_PARAMETERS_PROTOCOL and functions for creation,
   manipulation, and initialization of EFI_SHELL_PARAMETERS_PROTOCOL.
 
-  Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
-  Copyright (c) 2009 - 2013, Intel Corporation. All rights reserved.<BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  (C) Copyright 2016 Hewlett Packard Enterprise Development LP<BR>
+  Copyright (C) 2014, Red Hat, Inc.
+  (C) Copyright 2013 Hewlett-Packard Development Company, L.P.<BR>
+  Copyright (c) 2009 - 2018, Intel Corporation. All rights reserved.<BR>
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
 #include "Shell.h"
 
+BOOLEAN AsciiRedirection = FALSE;
+
 /**
-  return the next parameter from a command line string;
+  Return the next parameter's end from a command line string.
+
+  @param[in] String        the string to parse
+**/
+CONST CHAR16*
+FindEndOfParameter(
+  IN CONST CHAR16 *String
+  )
+{
+  CONST CHAR16 *First;
+  CONST CHAR16 *CloseQuote;
+
+  First = FindFirstCharacter(String, L" \"", L'^');
+
+  //
+  // nothing, all one parameter remaining
+  //
+  if (*First == CHAR_NULL) {
+    return (First);
+  }
+
+  //
+  // If space before a quote (or neither found, i.e. both CHAR_NULL),
+  // then that's the end.
+  //
+  if (*First == L' ') {
+    return (First);
+  }
+
+  CloseQuote = FindFirstCharacter (First+1, L"\"", L'^');
+
+  //
+  // We did not find a terminator...
+  //
+  if (*CloseQuote == CHAR_NULL) {
+    return (NULL);
+  }
+
+  return (FindEndOfParameter (CloseQuote+1));
+}
+
+/**
+  Return the next parameter from a command line string.
 
   This function moves the next parameter from Walker into TempParameter and moves
   Walker up past that parameter for recursive calling.  When the final parameter
@@ -26,25 +66,36 @@
   Temp Parameter must be large enough to hold the parameter before calling this
   function.
 
-  @param[in, out] Walker        pointer to string of command line.  Adjusted to
-                                reminaing command line on return
-  @param[in, out] TempParameter pointer to string of command line item extracted.
+  This will also remove all remaining ^ characters after processing.
 
+  @param[in, out] Walker          pointer to string of command line.  Adjusted to
+                                  remaining command line on return
+  @param[in, out] TempParameter   pointer to string of command line item extracted.
+  @param[in]      Length          buffer size of TempParameter.
+  @param[in]      StripQuotation  if TRUE then strip the quotation marks surrounding
+                                  the parameters.
+
+  @return   EFI_INVALID_PARAMETER A required parameter was NULL or pointed to a NULL or empty string.
+  @return   EFI_NOT_FOUND         A closing " could not be found on the specified string
 **/
-VOID
-EFIAPI
+EFI_STATUS
 GetNextParameter(
-  CHAR16 **Walker,
-  CHAR16 **TempParameter
+  IN OUT CHAR16   **Walker,
+  IN OUT CHAR16   **TempParameter,
+  IN CONST UINTN  Length,
+  IN BOOLEAN      StripQuotation
   )
 {
-  CHAR16 *NextDelim;
-  CHAR16 *TempLoc;
+  CONST CHAR16 *NextDelim;
+
+  if (Walker           == NULL
+    ||*Walker          == NULL
+    ||TempParameter    == NULL
+    ||*TempParameter   == NULL
+    ){
+    return (EFI_INVALID_PARAMETER);
+  }
 
-  ASSERT(Walker           != NULL);
-  ASSERT(*Walker          != NULL);
-  ASSERT(TempParameter    != NULL);
-  ASSERT(*TempParameter   != NULL);
 
   //
   // make sure we dont have any leading spaces
@@ -57,77 +108,64 @@ GetNextParameter(
   // make sure we still have some params now...
   //
   if (StrLen(*Walker) == 0) {
-    ASSERT((*Walker)[0] == CHAR_NULL);
-    *Walker = NULL;
-    return;
+DEBUG_CODE_BEGIN();
+    *Walker        = NULL;
+DEBUG_CODE_END();
+    return (EFI_INVALID_PARAMETER);
+  }
+
+  NextDelim = FindEndOfParameter(*Walker);
+
+  if (NextDelim == NULL){
+DEBUG_CODE_BEGIN();
+    *Walker        = NULL;
+DEBUG_CODE_END();
+    return (EFI_NOT_FOUND);
+  }
+
+  StrnCpyS(*TempParameter, Length / sizeof(CHAR16), (*Walker), NextDelim - *Walker);
+
+  //
+  // Add a CHAR_NULL if we didn't get one via the copy
+  //
+  if (*NextDelim != CHAR_NULL) {
+    (*TempParameter)[NextDelim - *Walker] = CHAR_NULL;
   }
 
   //
-  // we have a quoted parameter
-  // could be the last parameter, but SHOULD have a trailing quote
+  // Update Walker for the next iteration through the function
   //
-  if ((*Walker)[0] == L'\"') {
-    NextDelim = NULL;
-    for (TempLoc = *Walker + 1 ; TempLoc != NULL && *TempLoc != CHAR_NULL ; TempLoc++) {
-      if (*TempLoc == L'^' && *(TempLoc+1) == L'\"') {
-        TempLoc++;
-      } else if (*TempLoc == L'\"') {
-        NextDelim = TempLoc;
-        break;
-      }
-    }
+  *Walker = (CHAR16*)NextDelim;
+
+  //
+  // Remove any non-escaped quotes in the string
+  // Remove any remaining escape characters in the string
+  //
+  for (NextDelim = FindFirstCharacter(*TempParameter, L"\"^", CHAR_NULL)
+    ; *NextDelim != CHAR_NULL
+    ; NextDelim = FindFirstCharacter(NextDelim, L"\"^", CHAR_NULL)
+    ) {
+    if (*NextDelim == L'^') {
 
-    if (NextDelim - ((*Walker)+1) == 0) {
       //
-      // found ""
+      // eliminate the escape ^
       //
-      StrCpy(*TempParameter, L"");
-      *Walker = NextDelim + 1;
-    } else if (NextDelim != NULL) {
-      StrnCpy(*TempParameter, (*Walker)+1, NextDelim - ((*Walker)+1));
-      *Walker = NextDelim + 1;
-    } else {
+      CopyMem ((CHAR16*)NextDelim, NextDelim + 1, StrSize (NextDelim + 1));
+      NextDelim++;
+    } else if (*NextDelim == L'\"') {
+
       //
-      // last one... someone forgot the training quote!
+      // eliminate the unescaped quote
       //
-      StrCpy(*TempParameter, *Walker);
-      *Walker = NULL;
+      if (StripQuotation) {
+        CopyMem ((CHAR16*)NextDelim, NextDelim + 1, StrSize (NextDelim + 1));
+    } else{
+        NextDelim++;
     }
-    for (TempLoc = *TempParameter ; TempLoc != NULL && *TempLoc != CHAR_NULL ; TempLoc++) {
-      if (*TempLoc == L'^' && *(TempLoc+1) == L'\"') {
-        CopyMem(TempLoc, TempLoc+1, StrSize(TempLoc) - sizeof(TempLoc[0]));
-      }
-    }
-  } else {
-    //
-    // we have a regular parameter (no quote) OR
-    // we have the final parameter (no trailing space)
-    //
-    NextDelim = StrStr((*Walker), L" ");
-    if (NextDelim != NULL) {
-      StrnCpy(*TempParameter, *Walker, NextDelim - (*Walker));
-      (*TempParameter)[NextDelim - (*Walker)] = CHAR_NULL;
-      *Walker = NextDelim+1;
-    } else {
-      //
-      // last one.
-      //
-      StrCpy(*TempParameter, *Walker);
-      *Walker = NULL;
-    }
-    for (NextDelim = *TempParameter ; NextDelim != NULL && *NextDelim != CHAR_NULL ; NextDelim++) {
-      if (*NextDelim == L'^' && *(NextDelim+1) == L'^') {
-        CopyMem(NextDelim, NextDelim+1, StrSize(NextDelim) - sizeof(NextDelim[0]));
-      }
-    }
-    while ((*TempParameter)[StrLen(*TempParameter)-1] == L' ') {
-      (*TempParameter)[StrLen(*TempParameter)-1] = CHAR_NULL;
-    }
-    while ((*TempParameter)[0] == L' ') {
-      CopyMem(*TempParameter, (*TempParameter)+1, StrSize(*TempParameter) - sizeof((*TempParameter)[0]));
     }
   }
-  return;
+
+  return EFI_SUCCESS;
 }
 
 /**
@@ -137,26 +175,34 @@ GetNextParameter(
   parameters for inclusion in EFI_SHELL_PARAMETERS_PROTOCOL.  this supports space
   delimited and quote surrounded parameter definition.
 
-  @param[in] CommandLine         String of command line to parse
-  @param[in, out] Argv           pointer to array of strings; one for each parameter
-  @param[in, out] Argc           pointer to number of strings in Argv array
+  All special character processing (alias, environment variable, redirection,
+  etc... must be complete before calling this API.
 
-  @return EFI_SUCCESS           the operation was sucessful
+  @param[in] CommandLine          String of command line to parse
+  @param[in] StripQuotation       if TRUE then strip the quotation marks surrounding
+                                  the parameters.
+  @param[in, out] Argv            pointer to array of strings; one for each parameter
+  @param[in, out] Argc            pointer to number of strings in Argv array
+
+  @return EFI_SUCCESS           the operation was successful
+  @return EFI_INVALID_PARAMETER some parameters are invalid
   @return EFI_OUT_OF_RESOURCES  a memory allocation failed.
 **/
 EFI_STATUS
-EFIAPI
 ParseCommandLineToArgs(
   IN CONST CHAR16 *CommandLine,
-  IN OUT CHAR16 ***Argv,
-  IN OUT UINTN *Argc
+  IN BOOLEAN      StripQuotation,
+  IN OUT CHAR16   ***Argv,
+  IN OUT UINTN    *Argc
   )
 {
   UINTN       Count;
   CHAR16      *TempParameter;
   CHAR16      *Walker;
   CHAR16      *NewParam;
+  CHAR16      *NewCommandLine;
   UINTN       Size;
+  EFI_STATUS  Status;
 
   ASSERT(Argc != NULL);
   ASSERT(Argv != NULL);
@@ -167,49 +213,62 @@ ParseCommandLineToArgs(
     return (EFI_SUCCESS);
   }
 
-  Size = StrSize(CommandLine);
+  NewCommandLine = AllocateCopyPool(StrSize(CommandLine), CommandLine);
+  if (NewCommandLine == NULL){
+    return (EFI_OUT_OF_RESOURCES);
+  }
+
+  TrimSpaces(&NewCommandLine);
+  Size = StrSize(NewCommandLine);
   TempParameter = AllocateZeroPool(Size);
   if (TempParameter == NULL) {
+    SHELL_FREE_NON_NULL(NewCommandLine);
     return (EFI_OUT_OF_RESOURCES);
   }
 
   for ( Count = 0
-      , Walker = (CHAR16*)CommandLine
+      , Walker = (CHAR16*)NewCommandLine
       ; Walker != NULL && *Walker != CHAR_NULL
-      ; GetNextParameter(&Walker, &TempParameter)
-      , Count++
-     );
-
-/*  Count = 0;
-  Walker = (CHAR16*)CommandLine;
-  while(Walker != NULL) {
-    GetNextParameter(&Walker, &TempParameter);
-    Count++;
+      ; Count++
+      ) {
+    if (EFI_ERROR(GetNextParameter(&Walker, &TempParameter, Size, TRUE))) {
+      break;
+    }
   }
-*/
+
   //
   // lets allocate the pointer array
   //
   (*Argv) = AllocateZeroPool((Count)*sizeof(CHAR16*));
   if (*Argv == NULL) {
-    SHELL_FREE_NON_NULL(TempParameter);
-    return (EFI_OUT_OF_RESOURCES);
+    Status = EFI_OUT_OF_RESOURCES;
+    goto Done;
   }
 
   *Argc = 0;
-  Walker = (CHAR16*)CommandLine;
+  Walker = (CHAR16*)NewCommandLine;
   while(Walker != NULL && *Walker != CHAR_NULL) {
     SetMem16(TempParameter, Size, CHAR_NULL);
-    GetNextParameter(&Walker, &TempParameter);
-    NewParam = AllocateZeroPool(StrSize(TempParameter));
-    ASSERT(NewParam != NULL);
-    StrCpy(NewParam, TempParameter);
+    if (EFI_ERROR(GetNextParameter(&Walker, &TempParameter, Size, StripQuotation))) {
+      Status = EFI_INVALID_PARAMETER;
+      goto Done;
+    }
+
+    NewParam = AllocateCopyPool(StrSize(TempParameter), TempParameter);
+    if (NewParam == NULL){
+      Status = EFI_OUT_OF_RESOURCES;
+      goto Done;
+    }
     ((CHAR16**)(*Argv))[(*Argc)] = NewParam;
     (*Argc)++;
   }
   ASSERT(Count >= (*Argc));
+  Status = EFI_SUCCESS;
+
+Done:
   SHELL_FREE_NON_NULL(TempParameter);
-  return (EFI_SUCCESS);
+  SHELL_FREE_NON_NULL(NewCommandLine);
+  return (Status);
 }
 
 /**
@@ -229,7 +288,6 @@ ParseCommandLineToArgs(
   @sa ParseCommandLineToArgs
 **/
 EFI_STATUS
-EFIAPI
 CreatePopulateInstallShellParametersProtocol (
   IN OUT EFI_SHELL_PARAMETERS_PROTOCOL  **NewShellParameters,
   IN OUT BOOLEAN                        *RootShellInstance
@@ -319,6 +377,7 @@ CreatePopulateInstallShellParametersProtocol (
     // Populate Argc and Argv
     //
     Status = ParseCommandLineToArgs(FullCommandLine,
+                                    TRUE,
                                     &(*NewShellParameters)->Argv,
                                     &(*NewShellParameters)->Argc);
 
@@ -358,7 +417,7 @@ CreatePopulateInstallShellParametersProtocol (
 }
 
 /**
-  frees all memory used by createion and installation of shell parameters protocol
+  frees all memory used by creation and installation of shell parameters protocol
   and if there was an old version installed it will restore that one.
 
   @param NewShellParameters the interface of EFI_SHELL_PARAMETERS_PROTOCOL that is
@@ -370,7 +429,6 @@ CreatePopulateInstallShellParametersProtocol (
   @sa UninstallProtocolInterface
 **/
 EFI_STATUS
-EFIAPI
 CleanUpShellParametersProtocol (
   IN OUT EFI_SHELL_PARAMETERS_PROTOCOL  *NewShellParameters
   )
@@ -409,7 +467,7 @@ CleanUpShellParametersProtocol (
 }
 
 /**
-  Determin if a file name represents a unicode file.
+  Determine if a file name represents a unicode file.
 
   @param[in] FileName     Pointer to the filename to open.
 
@@ -417,7 +475,6 @@ CleanUpShellParametersProtocol (
   @return An error upon failure.
 **/
 EFI_STATUS
-EFIAPI
 IsUnicodeFile(
   IN CONST CHAR16 *FileName
   )
@@ -441,7 +498,7 @@ IsUnicodeFile(
   }
   gEfiShellProtocol->SetFilePosition(Handle, OriginalFilePosition);
   gEfiShellProtocol->CloseFile(Handle);
-  return (Status);  
+  return (Status);
 }
 
 /**
@@ -452,7 +509,6 @@ IsUnicodeFile(
   @param[in, out] TheString  A pointer to the string to update.
 **/
 VOID
-EFIAPI
 StripQuotes (
   IN OUT CHAR16 *TheString
   )
@@ -471,7 +527,7 @@ StripQuotes (
 }
 
 /**
-  Calcualte the 32-bit CRC in a EFI table using the service provided by the
+  Calculate the 32-bit CRC in a EFI table using the service provided by the
   gRuntime service.
 
   @param  Hdr                    Pointer to an EFI standard header
@@ -504,7 +560,6 @@ CalculateEfiHdrCrc (
   @return       The modified FileName.
 **/
 CHAR16*
-EFIAPI
 FixFileName (
   IN CHAR16 *FileName
   )
@@ -520,7 +575,7 @@ FixFileName (
     Copy = FileName+1;
     if ((TempLocation = StrStr(Copy , L"\"")) != NULL) {
       TempLocation[0] = CHAR_NULL;
-    }    
+    }
   } else {
     Copy = FileName;
     while(Copy[0] == L' ') {
@@ -528,7 +583,7 @@ FixFileName (
     }
     if ((TempLocation = StrStr(Copy , L" ")) != NULL) {
       TempLocation[0] = CHAR_NULL;
-    }    
+    }
   }
 
   if (Copy[0] == CHAR_NULL) {
@@ -539,7 +594,65 @@ FixFileName (
 }
 
 /**
-  Funcion will replace the current StdIn and StdOut in the ShellParameters protocol
+  Fix a string to only have the environment variable name, removing starting at the first space of whatever is quoted and removing the leading and trailing %.
+
+  @param[in]  FileName    The filename to start with.
+
+  @retval NULL  FileName was invalid.
+  @return       The modified FileName.
+**/
+CHAR16*
+FixVarName (
+  IN CHAR16 *FileName
+  )
+{
+  CHAR16  *Copy;
+  CHAR16  *TempLocation;
+
+  Copy = FileName;
+
+  if (FileName[0] == L'%') {
+    Copy = FileName+1;
+    if ((TempLocation = StrStr(Copy , L"%")) != NULL) {
+      TempLocation[0] = CHAR_NULL;
+    }
+  }
+
+  return (FixFileName(Copy));
+}
+
+
+/**
+  Write the unicode file tag to the specified file.
+
+  It is the caller's responsibility to ensure that
+  ShellInfoObject.NewEfiShellProtocol has been initialized before calling this
+  function.
+
+  @param[in] FileHandle  The file to write the unicode file tag to.
+
+  @return  Status code from ShellInfoObject.NewEfiShellProtocol->WriteFile.
+**/
+EFI_STATUS
+WriteFileTag (
+  IN SHELL_FILE_HANDLE FileHandle
+  )
+{
+  CHAR16     FileTag;
+  UINTN      Size;
+  EFI_STATUS Status;
+
+  FileTag = gUnicodeFileTag;
+  Size = sizeof FileTag;
+  Status = ShellInfoObject.NewEfiShellProtocol->WriteFile (FileHandle, &Size,
+                                                  &FileTag);
+  ASSERT (EFI_ERROR (Status) || Size == sizeof FileTag);
+  return Status;
+}
+
+
+/**
+  Function will replace the current StdIn and StdOut in the ShellParameters protocol
   structure by parsing NewCommandLine.  The current values are returned to the
   user.
 
@@ -552,11 +665,10 @@ FixFileName (
   @param[out] OldStdErr                  Pointer to old StdErr.
   @param[out] SystemTableInfo            Pointer to old system table information.
 
-  @retval   EFI_SUCCESS                 Operation was sucessful, Argv and Argc are valid.
+  @retval   EFI_SUCCESS                 Operation was successful, Argv and Argc are valid.
   @retval   EFI_OUT_OF_RESOURCES        A memory allocation failed.
 **/
 EFI_STATUS
-EFIAPI
 UpdateStdInStdOutStdErr(
   IN OUT EFI_SHELL_PARAMETERS_PROTOCOL  *ShellParameters,
   IN CHAR16                             *NewCommandLine,
@@ -583,12 +695,13 @@ UpdateStdInStdOutStdErr(
   BOOLEAN           OutAppend;
   BOOLEAN           ErrAppend;
   UINTN             Size;
-  CHAR16            TagBuffer[2];
   SPLIT_LIST        *Split;
   CHAR16            *FirstLocation;
+  BOOLEAN           Volatile;
 
   OutUnicode      = TRUE;
   InUnicode       = TRUE;
+  AsciiRedirection = FALSE;
   ErrUnicode      = TRUE;
   StdInVarName    = NULL;
   StdOutVarName   = NULL;
@@ -695,7 +808,7 @@ UpdateStdInStdOutStdErr(
     if (StrStr(CommandLineWalker, L" 1>> ") != NULL) {
       Status = EFI_NOT_FOUND;
     }
-  } 
+  }
   if (!EFI_ERROR(Status) && (CommandLineWalker = StrStr(CommandLineCopy, L" >> ")) != NULL) {
     FirstLocation = MIN(CommandLineWalker, FirstLocation);
     SetMem16(CommandLineWalker, 8, L' ');
@@ -722,7 +835,7 @@ UpdateStdInStdOutStdErr(
     if (StrStr(CommandLineWalker, L" >>a ") != NULL) {
       Status = EFI_NOT_FOUND;
     }
-  } 
+  }
   if (!EFI_ERROR(Status) && (CommandLineWalker = StrStr(CommandLineCopy, L" 1>a ")) != NULL) {
     FirstLocation = MIN(CommandLineWalker, FirstLocation);
     SetMem16(CommandLineWalker, 10, L' ');
@@ -736,7 +849,7 @@ UpdateStdInStdOutStdErr(
     if (StrStr(CommandLineWalker, L" 1>a ") != NULL) {
       Status = EFI_NOT_FOUND;
     }
-  } 
+  }
   if (!EFI_ERROR(Status) && (CommandLineWalker = StrStr(CommandLineCopy, L" >a ")) != NULL) {
     FirstLocation = MIN(CommandLineWalker, FirstLocation);
     SetMem16(CommandLineWalker, 8, L' ');
@@ -867,6 +980,7 @@ UpdateStdInStdOutStdErr(
     } else {
       StdInFileName   = CommandLineWalker += 4;
       InUnicode       = FALSE;
+      AsciiRedirection = TRUE;
     }
     if (StrStr(CommandLineWalker, L" <a ") != NULL) {
       Status = EFI_NOT_FOUND;
@@ -888,12 +1002,12 @@ UpdateStdInStdOutStdErr(
   //
   // re-populate the string to support any filenames that were in quotes.
   //
-  StrCpy(CommandLineCopy, NewCommandLine);
+  StrnCpyS(CommandLineCopy, StrSize(CommandLineCopy)/sizeof(CHAR16), NewCommandLine, StrLen(NewCommandLine));
 
   if (FirstLocation != CommandLineCopy + StrLen(CommandLineCopy)
-    && ((UINTN)(FirstLocation - CommandLineCopy) < StrLen(NewCommandLine))
+    && (((UINTN)FirstLocation - (UINTN)CommandLineCopy)/sizeof(CHAR16) < StrLen(NewCommandLine))
     ){
-    *(NewCommandLine + (UINTN)(FirstLocation - CommandLineCopy)) = CHAR_NULL;
+    *(NewCommandLine + ((UINTN)FirstLocation - (UINTN)CommandLineCopy)/sizeof(CHAR16)) = CHAR_NULL;
   }
 
   if (!EFI_ERROR(Status)) {
@@ -914,17 +1028,17 @@ UpdateStdInStdOutStdErr(
       }
     }
     if (StdErrVarName  != NULL) {
-      if ((StdErrVarName     = FixFileName(StdErrVarName)) == NULL) {
+      if ((StdErrVarName     = FixVarName(StdErrVarName)) == NULL) {
         Status = EFI_INVALID_PARAMETER;
       }
     }
     if (StdOutVarName  != NULL) {
-      if ((StdOutVarName     = FixFileName(StdOutVarName)) == NULL) {
+      if ((StdOutVarName     = FixVarName(StdOutVarName)) == NULL) {
         Status = EFI_INVALID_PARAMETER;
       }
     }
     if (StdInVarName   != NULL) {
-      if ((StdInVarName      = FixFileName(StdInVarName)) == NULL) {
+      if ((StdInVarName      = FixVarName(StdInVarName)) == NULL) {
         Status = EFI_INVALID_PARAMETER;
       }
     }
@@ -959,12 +1073,12 @@ UpdateStdInStdOutStdErr(
       //
       // Check for no volatile environment variables
       //
-      ||(StdErrVarName  != NULL && !IsVolatileEnv(StdErrVarName))
-      ||(StdOutVarName  != NULL && !IsVolatileEnv(StdOutVarName))
+      ||(StdErrVarName  != NULL && !EFI_ERROR (IsVolatileEnv (StdErrVarName, &Volatile)) && !Volatile)
+      ||(StdOutVarName  != NULL && !EFI_ERROR (IsVolatileEnv (StdOutVarName, &Volatile)) && !Volatile)
       //
       // Cant redirect during a reconnect operation.
       //
-      ||(StrStr(NewCommandLine, L"connect -r") != NULL 
+      ||(StrStr(NewCommandLine, L"connect -r") != NULL
          && (StdOutVarName != NULL || StdOutFileName != NULL || StdErrFileName != NULL || StdErrVarName != NULL))
       //
       // Check that filetypes (Unicode/Ascii) do not change during an append
@@ -995,13 +1109,7 @@ UpdateStdInStdOutStdErr(
         }
         Status = ShellOpenFileByName(StdErrFileName, &TempHandle, EFI_FILE_MODE_WRITE|EFI_FILE_MODE_READ|EFI_FILE_MODE_CREATE,0);
         if (!ErrAppend && ErrUnicode && !EFI_ERROR(Status)) {
-          //
-          // Write out the gUnicodeFileTag
-          //
-          Size = sizeof(CHAR16);
-          TagBuffer[0] = gUnicodeFileTag;
-          TagBuffer[1] = CHAR_NULL;
-          ShellInfoObject.NewEfiShellProtocol->WriteFile(TempHandle, &Size, TagBuffer);
+          Status = WriteFileTag (TempHandle);
         }
         if (!ErrUnicode && !EFI_ERROR(Status)) {
           TempHandle = CreateFileInterfaceFile(TempHandle, FALSE);
@@ -1027,23 +1135,23 @@ UpdateStdInStdOutStdErr(
         if (TempHandle == NULL) {
           Status = EFI_INVALID_PARAMETER;
         } else {
-          if (StrStr(StdOutFileName, L"NUL")==StdOutFileName) {
+          if (gUnicodeCollation->MetaiMatch (gUnicodeCollation, StdOutFileName, L"NUL")) {
             //no-op
           } else if (!OutAppend && OutUnicode && !EFI_ERROR(Status)) {
-            //
-            // Write out the gUnicodeFileTag
-            //
-            Size = sizeof(CHAR16);
-            TagBuffer[0] = gUnicodeFileTag;
-            TagBuffer[1] = CHAR_NULL;
-            ShellInfoObject.NewEfiShellProtocol->WriteFile(TempHandle, &Size, TagBuffer);
+            Status = WriteFileTag (TempHandle);
           } else if (OutAppend) {
-            //
-            // Move to end of file
-            //
             Status = ShellInfoObject.NewEfiShellProtocol->GetFileSize(TempHandle, &FileSize);
             if (!EFI_ERROR(Status)) {
-              Status = ShellInfoObject.NewEfiShellProtocol->SetFilePosition(TempHandle, FileSize);
+              //
+              // When appending to a new unicode file, write the file tag.
+              // Otherwise (ie. when appending to a new ASCII file, or an
+              // existent file with any encoding), just seek to the end.
+              //
+              Status = (FileSize == 0 && OutUnicode) ?
+                         WriteFileTag (TempHandle) :
+                         ShellInfoObject.NewEfiShellProtocol->SetFilePosition (
+                                                                TempHandle,
+                                                                FileSize);
             }
           }
           if (!OutUnicode && !EFI_ERROR(Status)) {
@@ -1119,10 +1227,13 @@ UpdateStdInStdOutStdErr(
           &TempHandle,
           EFI_FILE_MODE_READ,
           0);
-        if (!InUnicode && !EFI_ERROR(Status)) {
-          TempHandle = CreateFileInterfaceFile(TempHandle, FALSE);
-        }
         if (!EFI_ERROR(Status)) {
+          if (!InUnicode) {
+            //
+            // Create the ASCII->Unicode conversion layer
+            //
+            TempHandle = CreateFileInterfaceFile(TempHandle, FALSE);
+          }
           ShellParameters->StdIn = TempHandle;
           gST->ConIn = CreateSimpleTextInOnFile(TempHandle, &gST->ConsoleInHandle);
         }
@@ -1134,13 +1245,20 @@ UpdateStdInStdOutStdErr(
   CalculateEfiHdrCrc(&gST->Hdr);
 
   if (gST->ConIn == NULL ||gST->ConOut == NULL) {
-    return (EFI_OUT_OF_RESOURCES);
+    Status = EFI_OUT_OF_RESOURCES;
   }
+
+  if (Status == EFI_NOT_FOUND) {
+    ShellPrintHiiEx(-1, -1, NULL, STRING_TOKEN (STR_SHELL_REDUNDA_REDIR), ShellInfoObject.HiiHandle);
+  } else if (EFI_ERROR(Status)) {
+    ShellPrintHiiEx(-1, -1, NULL, STRING_TOKEN (STR_SHELL_INVALID_REDIR), ShellInfoObject.HiiHandle);
+  }
+
   return (Status);
 }
 
 /**
-  Funcion will replace the current StdIn and StdOut in the ShellParameters protocol
+  Function will replace the current StdIn and StdOut in the ShellParameters protocol
   structure with StdIn and StdOut.  The current values are de-allocated.
 
   @param[in, out] ShellParameters      Pointer to parameter structure to modify.
@@ -1150,7 +1268,6 @@ UpdateStdInStdOutStdErr(
   @param[in] SystemTableInfo           Pointer to old system table information.
 **/
 EFI_STATUS
-EFIAPI
 RestoreStdInStdOutStdErr (
   IN OUT EFI_SHELL_PARAMETERS_PROTOCOL  *ShellParameters,
   IN  SHELL_FILE_HANDLE                 *OldStdIn,
@@ -1161,7 +1278,7 @@ RestoreStdInStdOutStdErr (
 {
   SPLIT_LIST        *Split;
 
-  if (ShellParameters == NULL 
+  if (ShellParameters == NULL
     ||OldStdIn        == NULL
     ||OldStdOut       == NULL
     ||OldStdErr       == NULL
@@ -1211,7 +1328,7 @@ RestoreStdInStdOutStdErr (
   return (EFI_SUCCESS);
 }
 /**
-  Funcion will replace the current Argc and Argv in the ShellParameters protocol
+  Function will replace the current Argc and Argv in the ShellParameters protocol
   structure by parsing NewCommandLine.  The current values are returned to the
   user.
 
@@ -1219,22 +1336,28 @@ RestoreStdInStdOutStdErr (
 
   @param[in, out] ShellParameters        Pointer to parameter structure to modify.
   @param[in] NewCommandLine              The new command line to parse and use.
+  @param[in] Type                        The type of operation.
   @param[out] OldArgv                    Pointer to old list of parameters.
   @param[out] OldArgc                    Pointer to old number of items in Argv list.
 
-  @retval   EFI_SUCCESS                 Operation was sucessful, Argv and Argc are valid.
+
+  @retval   EFI_SUCCESS                 Operation was successful, Argv and Argc are valid.
+  @return   EFI_INVALID_PARAMETER       Some parameters are invalid.
   @retval   EFI_OUT_OF_RESOURCES        A memory allocation failed.
 **/
 EFI_STATUS
-EFIAPI
 UpdateArgcArgv(
   IN OUT EFI_SHELL_PARAMETERS_PROTOCOL  *ShellParameters,
   IN CONST CHAR16                       *NewCommandLine,
+  IN SHELL_OPERATION_TYPES              Type,
   OUT CHAR16                            ***OldArgv OPTIONAL,
   OUT UINTN                             *OldArgc OPTIONAL
   )
 {
+  BOOLEAN                 StripParamQuotation;
+
   ASSERT(ShellParameters != NULL);
+  StripParamQuotation = TRUE;
 
   if (OldArgc != NULL) {
     *OldArgc = ShellParameters->Argc;
@@ -1243,11 +1366,19 @@ UpdateArgcArgv(
     *OldArgv = ShellParameters->Argv;
   }
 
-  return (ParseCommandLineToArgs(NewCommandLine, &(ShellParameters->Argv), &(ShellParameters->Argc)));
+  if (Type == Script_File_Name) {
+    StripParamQuotation = FALSE;
+  }
+
+  return ParseCommandLineToArgs( NewCommandLine,
+                                 StripParamQuotation,
+                                 &(ShellParameters->Argv),
+                                 &(ShellParameters->Argc)
+                                );
 }
 
 /**
-  Funcion will replace the current Argc and Argv in the ShellParameters protocol
+  Function will replace the current Argc and Argv in the ShellParameters protocol
   structure with Argv and Argc.  The current values are de-allocated and the
   OldArgv must not be deallocated by the caller.
 
@@ -1256,7 +1387,6 @@ UpdateArgcArgv(
   @param[in] OldArgc                    pointer to old number of items in Argv list
 **/
 VOID
-EFIAPI
 RestoreArgcArgv(
   IN OUT EFI_SHELL_PARAMETERS_PROTOCOL  *ShellParameters,
   IN CHAR16                             ***OldArgv,
