@@ -2,13 +2,7 @@
   Implements editor interface functions.
 
   Copyright (c) 2005 - 2018, Intel Corporation. All rights reserved. <BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -46,7 +40,7 @@ EFI_EDITOR_GLOBAL_EDITOR      MainEditor;
   Load a file from disk to editor
 
   @retval EFI_SUCCESS             The operation was successful.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
 **/
 EFI_STATUS
@@ -80,7 +74,7 @@ MainCommandGotoLine (
 
   @retval EFI_SUCCESS           The file was saved correctly.
   @retval EFI_OUT_OF_RESOURCES  A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A file access error occured.
+  @retval EFI_LOAD_ERROR          A file access error occurred.
 **/
 EFI_STATUS
 MainCommandSaveFile (
@@ -102,7 +96,7 @@ MainCommandDisplayHelp (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandExit (
@@ -114,7 +108,7 @@ MainCommandExit (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandSearch (
@@ -126,7 +120,7 @@ MainCommandSearch (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandSearchReplace (
@@ -138,7 +132,7 @@ MainCommandSearchReplace (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandCutLine (
@@ -150,7 +144,7 @@ MainCommandCutLine (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandPasteLine (
@@ -283,7 +277,7 @@ EDITOR_MENU_ITEM  MainMenuItems[] = {
   Load a file from disk to editor
 
   @retval EFI_SUCCESS             The operation was successful.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
 **/
 EFI_STATUS
@@ -471,7 +465,7 @@ MainCommandSwitchFileType (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandCutLine (
@@ -509,7 +503,7 @@ MainCommandCutLine (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandPasteLine (
@@ -541,7 +535,7 @@ MainCommandPasteLine (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandSearch (
@@ -682,7 +676,7 @@ MainCommandSearch (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandSearchReplace (
@@ -985,7 +979,7 @@ MainCommandSearchReplace (
 
   @retval EFI_SUCCESS             The operation was successful.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainCommandExit (
@@ -1139,7 +1133,7 @@ MainCommandGotoLine (
 
   @retval EFI_SUCCESS           The file was saved correctly.
   @retval EFI_OUT_OF_RESOURCES  A memory allocation failed.
-  @retval EFI_LOAD_ERROR          A file access error occured.
+  @retval EFI_LOAD_ERROR          A file access error occurred.
 **/
 EFI_STATUS
 MainCommandSaveFile (
@@ -1445,7 +1439,7 @@ EFI_EDITOR_GLOBAL_EDITOR      MainEditorConst = {
   The initialization function for MainEditor.
 
   @retval EFI_SUCCESS             The operation was successful.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainEditorInit (
@@ -1591,7 +1585,7 @@ MainEditorInit (
   The cleanup function for MainEditor.
 
   @retval EFI_SUCCESS             The operation was successful.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
 **/
 EFI_STATUS
 MainEditorCleanup (
@@ -1829,7 +1823,7 @@ MainEditorHandleMouseInput (
   Handle user key input. This routes to other functions for the actions.
 
   @retval EFI_SUCCESS             The operation was successful.
-  @retval EFI_LOAD_ERROR          A load error occured.
+  @retval EFI_LOAD_ERROR          A load error occurred.
   @retval EFI_OUT_OF_RESOURCES    A memory allocation failed.
 **/
 EFI_STATUS
